/* blendfile.c
 * Copyright (c) 2000 by Not a Number B.V. (NaN)
 * This file is covered by the license in the accompanying "LICENSE" file.
 * $Id: blendfile.c 74 2004-05-02 20:25:15Z goofster $
 */

#include <string.h>
#include <stdio.h>
#include "nspr.h"

#include "nspr-additions.h"

#include "blendfile.h"

extern int debug;

typedef struct BHeadType {
	int code, len;
	void *old;
	int SDNAnr, nr;
} BHead;

void scan_blendfile(char *name, int *startfr, int *endfr, char *scenename) {
	// sigh we need it differently here from the read_blenderfile in
	// broker.c
	BHead *bhead;
	int skipdata, *ip, found = FALSE;
	char *fd, *bfcopy;
	int filelen;

	PRFileDesc *fd_file;
	PRFileInfo fileinfo;
	PRInt32 BlendFileSize, haveread;

	if ((fd_file = PR_Open(name, PR_RDONLY, 0)) == NULL)
	{
		if (debug>0) fprintf(stderr, "scan_blendfile: cannot read %s : %s\n", name, PRB_GetErrorText());
		return;
	}
	PR_GetFileInfo(name, &fileinfo);
	BlendFileSize = fileinfo.size;
	filelen = BlendFileSize;

	if ((bfcopy = malloc(BlendFileSize)) == NULL)
	{
		if (debug>0) fprintf(stderr, "scan_blendfile: cannot malloc %d\n", BlendFileSize);
		PR_Close(fd_file);
		return;
	}
	if ((haveread = PR_Read(fd_file, bfcopy, BlendFileSize)) != BlendFileSize)
	{
		if (debug>0) fprintf(stderr, "scan_blendfile: only read %d of %d bytes from %s : %s\n", haveread, filelen, name, PRB_GetErrorText());
		PR_Close(fd_file);
		return;
	}
	PR_Close(fd_file);

	// in a "REND" : sfra, efra (len==8) and a scenename (len==32)

	fd = bfcopy;
	fd += 12;
	filelen -= 12;

	while ((found == FALSE) && (filelen>0))
	{
		char probe[5];
		bhead = (BHead *)fd;

		skipdata = bhead->len + sizeof(BHead);

		strncpy(probe, (char *)&(bhead->code), 4);
		probe[4] = 0;

		if (debug>0) fprintf(stdout, "scan_blendfile: %s length=%d probe=%s len=%d\n", name, filelen, probe, bhead->len);


		if (strncmp(probe, "REND", 4) == 0 )
		{
			ip = (int *) (fd + sizeof(BHead));

			*startfr = ip[0];
			*endfr = ip[1];
			scenename[0] = '\0';

			if (bhead->len == 8)
			{
				if (debug>1) fprintf(stdout, "QUEUE blender -b %s -s %d -e %d -a\n", name, *startfr, *endfr);
			}
			else if (bhead->len == 32)
			{ // other scene
				strcpy(scenename, (char *)(ip+2));
				if (debug>1) fprintf(stdout, "QUEUE blender -b %s -S %s -s %d -e %d -a\n", name, (char *)(ip+2), *startfr, *endfr);
			} else
			{
				if (debug>1) fprintf(stdout, "scan_blendfile: ERROR %d-%d bhead->len=%d\n", *startfr, *endfr, bhead->len);
			}
			found = TRUE;
		}
		fd += skipdata;
		filelen -= skipdata;
	}
	free(bfcopy);
}
