/* scan.c
 * Copyright (c) 2000 by Not a Number B.V. (NaN)
 * This file is covered by the license in the accompanying "LICENSE" file.
 * $Id: scan.c 86 2004-10-26 15:42:41Z goofster $
 */

#include <nspr.h>

#include "nspr-additions.h"
#include "scan.h"

#define MAXLINE    384

// another mutex candidate
extern int debug;

void PR_CALLBACK renderd_scan_thread(void *arg) {
	PRNetAddr scanAddr;
	int hostip,hostnet,broadcast,curip;
	int i;
	char hostname[SYS_INFO_BUFFER_LENGTH];
	char buffer[PR_NETDB_BUF_SIZE];
	char renderdType[SYS_INFO_BUFFER_LENGTH];
	PRHostEnt host;
	PRFileDesc *broadcastSock;
	struct twoports port;
	int nread;
	char buf2[MAXLINE];

	port = *(struct twoports *) arg;
	//get the host ip
	if (PR_GetSystemInfo(PR_SI_HOSTNAME, hostname, SYS_INFO_BUFFER_LENGTH)==PR_FAILURE)
		if (debug>1) fprintf(stderr, "PR_GetSystemInfo Hostname Failed: %s\n", PRB_GetErrorText());
	//get the host type
	if (PR_GetSystemInfo(PR_SI_SYSNAME, renderdType, SYS_INFO_BUFFER_LENGTH)==PR_FAILURE)
		if (debug>1) fprintf(stderr, "PR_GetSystemInfo Sysname Failed: %s\n", PRB_GetErrorText());
	//get the host by name
	if (PR_GetHostByName(hostname, buffer, PR_NETDB_BUF_SIZE, &host)==PR_FAILURE)
		if (debug>1) fprintf(stderr, "PR_GetHostByName Hostname %s Failed: %s\n", hostname, PRB_GetErrorText());

	if (debug>0) fprintf(stdout, "Checking on port #%i\n",port.remote);

	if (PR_EnumerateHostEnt(0, &host, port.remote, &scanAddr)<0)
		if (debug>1) fprintf(stderr, "PR_EnumerateHostEnt Failed: %s\n", PRB_GetErrorText());

	hostip = PR_ntohl(scanAddr.inet.ip);
	hostnet = hostip & 0xffffff00;
	broadcast = hostnet | 0xff;
	if (debug>1) fprintf(stdout, "host ip: %u.%u.%u.%u\n", BIP(hostip));

	if (debug>0) fprintf(stdout, "Scanning subnet for broker...\n");
	for (i=1; i<255; i++) {
		curip = hostnet+i;
		scanAddr.inet.ip = PR_htonl(curip);
		broadcastSock = PR_NewTCPSocket();
		sprintf(buf2, "%u.%u.%u.%u %u", BIP(curip), port.remote);
		if ( PR_Connect(broadcastSock, &scanAddr, BTO(2) )==PR_FAILURE ) {
			if (debug>1) fprintf(stderr,"PR_Connect failed: %s  %s (%ld)\n", buf2, PRB_GetErrorText(), (LONGINT) PR_GetOSError());
		} else {
			if (debug>0) fprintf(stdout, "found broker: %s\n", buf2);
		
			nread = PRB_ReadLineTime(broadcastSock, buffer, PR_NETDB_BUF_SIZE,BTO_SEC(5));		
			if (debug>1) fprintf(stdout, "readline: [%d]  '%s'\n", nread, buffer);
			
			PRB_Msg(broadcastSock, "NewRenderd %s %d", hostname, port.local);			
			if (debug>1) fprintf(stdout, "sent message 'NewRenderd %s %i' to broker\n", hostname, port.local);
			
			PRB_Msg(broadcastSock, "Renderd type: %s", renderdType);
			PRB_Msg(broadcastSock, "Quit");			
			if (debug>1) fprintf(stdout, "sent message 'Quit' to broker\n");

			nread = PRB_ReadLine(broadcastSock, buffer, PR_NETDB_BUF_SIZE);
			if (debug>1) fprintf(stdout, "readline: [%d]  '%s'\n", nread, buffer);
			
			PRB_Close(broadcastSock);
			break;
		}
		PRB_Close(broadcastSock);
	}
}

void PR_CALLBACK broker_scan_thread(void *arg) {
	PRNetAddr scanAddr, hostAddr;
	int hostip,hostnet,broadcast,curip;
	int i;
	char hostname[MAXLINE];
	char buffer[PR_NETDB_BUF_SIZE];
	PRHostEnt host;
	PRFileDesc *broadcastSock, *hostSock;
	struct twoports port;
	int nread;
	char buf2[MAXLINE];

	port = *(struct twoports *)arg;
	//get the host ip
	if (PR_GetSystemInfo(PR_SI_HOSTNAME, hostname, MAXLINE)==PR_FAILURE)
		if (debug>1) fprintf(stderr, "PR_GetSystemInfo Hostname Failed: %s\n", PRB_GetErrorText());
	
	if (PR_GetHostByName(hostname, buffer, PR_NETDB_BUF_SIZE, &host)==PR_FAILURE)
		if (debug>1) fprintf(stderr, "PR_GetHostByName Hostname %s Failed: %s\n", hostname, PRB_GetErrorText());
	
	PR_EnumerateHostEnt(0, &host, port.remote, &scanAddr);
	hostip = PR_ntohl(scanAddr.inet.ip);
	hostnet = hostip & 0xffffff00;
	broadcast = hostnet | 0xff;
	if (debug>1) fprintf(stdout, "host ip: %u.%u.%u.%u\n", BIP(hostip));

	// this hostSock stuff should not be needed. perhaps we should break
	// up scan.c in a broker and a renderd part ?
	hostSock = PR_NewTCPSocket();
	hostAddr = scanAddr;
	hostAddr.inet.port = PR_htons(port.local);
	if ( PR_Connect(hostSock, &hostAddr, PR_TicksPerSecond())==PR_FAILURE ){
		 if (debug>1) fprintf(stderr, "PR_Connect failed: %u.%u.%u.%u:%u  %s (%ld)\n",
			   BIP(hostip), port.local,
				   PRB_GetErrorText(), (LONGINT) PR_GetOSError());
		 PRB_Close(hostSock);
		 return;
	}

	nread = PRB_ReadLine(hostSock, buffer, PR_NETDB_BUF_SIZE);
	if (debug>1) fprintf(stdout, "readline: [%d]  '%s'\n", nread, buffer);

	if (debug>0) fprintf(stdout, "Scanning subnet for renderdaemons");
	for (i=1; i<255; i++) {
		curip = hostnet+i;
		scanAddr.inet.ip = PR_htonl(curip);
		broadcastSock = PR_NewTCPSocket();
		sprintf(buf2,"%u.%u.%u.%u %u", BIP(curip), port.remote);
		if ( PR_Connect(broadcastSock, &scanAddr, BTO(2) )==PR_FAILURE )
			fprintf(stderr, " PR_Connect failed: %s  %s (%ld)\n", buf2,
					   PRB_GetErrorText(), (LONGINT) PR_GetOSError());
		else {
			if (debug>0) fprintf(stdout, "found renderd: %s\n", buf2);
			PRB_Msg(broadcastSock, "Quit");
			if (debug>1) fprintf(stdout, "sent message Quit to renderd\n");
			nread = PRB_ReadLineTime(broadcastSock, buffer, PR_NETDB_BUF_SIZE,BTO_SEC(5));
			if (debug>1) fprintf(stdout, "readline: [%d]  '%s'\n", nread, buffer);
			PRB_Msg(hostSock,"NewRenderd %s", buf2);
			if (debug>1) fprintf(stdout, "sent message NewRenderd\n");
		}
		PRB_Close(broadcastSock);
	}
	PRB_Msg(hostSock, "Quit");
	if (debug>1) fprintf(stdout, "sent message 'Quit' to host\n");
	nread = PRB_ReadLine(hostSock, buffer, PR_NETDB_BUF_SIZE);
	if (debug>1) fprintf(stdout, "readline=%d  '%s'\n", nread, buffer);
	PRB_Close(hostSock);
	return;
}
